@@ -151,19 +151,15 @@ module.exports = grunt => {
                             //
                             // It also adds the Moodle plugin name to the AMD module definition
                             // so that it can be imported as expected in other modules.
-                            path.resolve('.grunt/babel-plugin-add-module-to-define.js'),
-                            '@babel/plugin-syntax-dynamic-import',
-                            '@babel/plugin-syntax-import-meta',
-                            ['@babel/plugin-proposal-class-properties', {'loose': false}],
-                            '@babel/plugin-proposal-json-strings'
+                            path.resolve('.grunt/babel-plugin-add-module-to-define.js')
                         ],
                         presets: [
                             ['@babel/preset-env', {
                                 targets: {
                                     browsers: [
-                                        ">0.25%",
+                                        ">0.3%",
                                         "last 2 versions",
-                                        "not ie <= 10",
+                                        "not ie >= 0",
                                         "not op_mini all",
                                         "not Opera > 0",
                                         "not dead"
@@ -203,6 +199,9 @@ module.exports = grunt => {
         },
     });
 
+    // Add the 'js' task as a startup task.
+    grunt.moodleEnv.startupTasks.push('js');
+
     // On watch, we dynamically modify config to build only affected files. This
     // method is slightly complicated to deal with multiple changed files at once (copied
     // from the grunt-contrib-watch readme).
