@@ -33,8 +33,8 @@ defined('MOODLE_INTERNAL') || die();
  * Otherwise it reimplements the same behaviour. See the PHP manual page for more info.
  *
  * @link http://php.net/manual/en/function.imagecopyresampled.php
- * @param resource $dst_img the destination GD image resource
- * @param resource $src_img the source GD image resource
+ * @param resource|\GdImage $dst_img the destination GD image resource
+ * @param resource|\GdImage $src_img the source GD image resource
  * @param int $dst_x vthe X coordinate of the upper left corner in the destination image
  * @param int $dst_y the Y coordinate of the upper left corner in the destination image
  * @param int $src_x the X coordinate of the upper left corner in the source image
@@ -48,7 +48,7 @@ defined('MOODLE_INTERNAL') || die();
 function imagecopybicubic($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y, $dst_w, $dst_h, $src_w, $src_h) {
     global $CFG;
 
-    if (function_exists('imagecopyresampled') and $CFG->gdversion >= 2) {
+    if (function_exists('imagecopyresampled')) {
        return imagecopyresampled($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y,
                                  $dst_w, $dst_h, $src_w, $src_h);
     }
@@ -88,27 +88,28 @@ function imagecopybicubic($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y, $d
 }
 
 /**
- * Stores optimised icon images in icon file area
+ * Stores optimised icon images in icon file area.
+ *
+ * Since 2.9 this function will generate an icon in the same format as the original file when possible.
+ * To counter that behaviour, you can use the argument $preferpng to generate a PNG icon.
  *
  * @param context $context
  * @param string $component
  * @param string filearea
  * @param int $itemid
  * @param string $originalfile
+ * @param boolean $preferpng When true, it will try to generate a PNG file regardless of the original file.
  * @return mixed new unique revision number or false if not saved
  */
-function process_new_icon($context, $component, $filearea, $itemid, $originalfile) {
+function process_new_icon($context, $component, $filearea, $itemid, $originalfile, $preferpng = false) {
     global $CFG;
 
-    if (empty($CFG->gdversion)) {
-        return false;
-    }
-
     if (!is_file($originalfile)) {
         return false;
     }
 
     $imageinfo = getimagesize($originalfile);
+    $imagefnc = '';
 
     if (empty($imageinfo)) {
         return false;
@@ -141,6 +142,13 @@ function process_new_icon($context, $component, $filearea, $itemid, $originalfil
                 debugging('JPEG not supported on this server');
                 return false;
             }
+            // If the user uploads a jpeg them we should process as a jpeg if possible.
+            if (!$preferpng && function_exists('imagejpeg')) {
+                $imagefnc = 'imagejpeg';
+                $imageext = '.jpg';
+                $filters = null; // Not used.
+                $quality = 90;
+            }
             break;
         case IMAGETYPE_PNG:
             if (function_exists('imagecreatefrompng')) {
@@ -154,22 +162,25 @@ function process_new_icon($context, $component, $filearea, $itemid, $originalfil
             return false;
     }
 
-    if (function_exists('imagepng')) {
-        $imagefnc = 'imagepng';
-        $imageext = '.png';
-        $filters = PNG_NO_FILTER;
-        $quality = 1;
-    } else if (function_exists('imagejpeg')) {
-        $imagefnc = 'imagejpeg';
-        $imageext = '.jpg';
-        $filters = null; // not used
-        $quality = 90;
-    } else {
-        debugging('Jpeg and png not supported on this server, please fix server configuration');
-        return false;
+    // The conversion has not been decided yet, let's apply defaults (png with fallback to jpg).
+    if (empty($imagefnc)) {
+        if (function_exists('imagepng')) {
+            $imagefnc = 'imagepng';
+            $imageext = '.png';
+            $filters = PNG_NO_FILTER;
+            $quality = 1;
+        } else if (function_exists('imagejpeg')) {
+            $imagefnc = 'imagejpeg';
+            $imageext = '.jpg';
+            $filters = null; // Not used.
+            $quality = 90;
+        } else {
+            debugging('Jpeg and png not supported on this server, please fix server configuration');
+            return false;
+        }
     }
 
-    if (function_exists('imagecreatetruecolor') and $CFG->gdversion >= 2) {
+    if (function_exists('imagecreatetruecolor')) {
         $im1 = imagecreatetruecolor(100, 100);
         $im2 = imagecreatetruecolor(35, 35);
         $im3 = imagecreatetruecolor(512, 512);
@@ -205,8 +216,8 @@ function process_new_icon($context, $component, $filearea, $itemid, $originalfil
         $im3 = imagecreate(512, 512);
     }
 
-    $cx = $image->width / 2;
-    $cy = $image->height / 2;
+    $cx = floor($image->width / 2);
+    $cy = floor($image->height / 2);
 
     if ($image->width < $image->height) {
         $half = floor($image->width / 2.0);
@@ -222,6 +233,14 @@ function process_new_icon($context, $component, $filearea, $itemid, $originalfil
 
     $icon = array('contextid'=>$context->id, 'component'=>$component, 'filearea'=>$filearea, 'itemid'=>$itemid, 'filepath'=>'/');
 
+    if ($imagefnc === 'imagejpeg') {
+        // Function imagejpeg() accepts less arguments than imagepng() but we need to make $imagefnc accept the same
+        // number of arguments, otherwise PHP8 throws an error.
+        $imagefnc = function($image, $filename, $quality, $unused) {
+            return imagejpeg($image, $filename, $quality);
+        };
+    }
+
     ob_start();
     if (!$imagefnc($im1, NULL, $quality, $filters)) {
         // keep old icons
@@ -259,40 +278,65 @@ function process_new_icon($context, $component, $filearea, $itemid, $originalfil
     return $file1->get_id();
 }
 
-/**
- * Generates a thumbnail for the given image
- *
- * If the GD library has at least version 2 and PNG support is available, the returned data
- * is the content of a transparent PNG file containing the thumbnail. Otherwise, the function
- * returns contents of a JPEG file with black background containing the thumbnail.
- *
- * @param string $filepath the full path to the original image file
- * @param int $width the width of the requested thumbnail
- * @param int $height the height of the requested thumbnail
- * @return string|bool false if a problem occurs, the thumbnail image data otherwise
- */
-function generate_image_thumbnail($filepath, $width, $height) {
-    global $CFG;
 
-    if (empty($CFG->gdversion) or empty($filepath) or empty($width) or empty($height)) {
+/**
+ * Resize an image from an image path.
+ *
+ * This maintains the aspect ratio of the image.
+ * This will not enlarge the image.
+ *
+ * @param string $filepath The full path to the original image file.
+ * @param int|null $width The max width of the resized image, or null to only use the height.
+ * @param int|null $height The max height of the resized image, or null to only use the width.
+ * @param bool $forcecanvas Whether the final dimensions should be set to $width and $height.
+ * @return string|bool False if a problem occurs, else the resized image data.
+ */
+function resize_image($filepath, $width, $height, $forcecanvas = false) {
+    if (empty($filepath)) {
         return false;
     }
 
-    $imageinfo = getimagesize($filepath);
+    // Fetch the image information for this image.
+    $imageinfo = @getimagesize($filepath);
+    if (empty($imageinfo)) {
+        return false;
+    }
+
+    // Create a new image from the file.
+    $original = @imagecreatefromstring(file_get_contents($filepath));
+
+    // Generate the thumbnail.
+    return resize_image_from_image($original, $imageinfo, $width, $height, $forcecanvas);
+}
+
+/**
+ * Resize an image from an image object.
+ *
+ * @param resource|\GdImage $original The image to work on.
+ * @param array $imageinfo Contains [0] => originalwidth, [1] => originalheight.
+ * @param int|null $width The max width of the resized image, or null to only use the height.
+ * @param int|null $height The max height of the resized image, or null to only use the width.
+ * @param bool $forcecanvas Whether the final dimensions should be set to $width and $height.
+ * @return string|bool False if a problem occurs, else the resized image data.
+ */
+function resize_image_from_image($original, $imageinfo, $width, $height, $forcecanvas = false) {
+    global $CFG;
+
+    if (empty($width) && empty($height) || ($forcecanvas && (empty($width) || empty($height)))) {
+        // We need do not have the required ddimensions to work with.
+        return false;
+    }
 
     if (empty($imageinfo)) {
         return false;
     }
 
-    $originalwidth = $imageinfo[0];
+    $originalwidth  = $imageinfo[0];
     $originalheight = $imageinfo[1];
-
     if (empty($originalwidth) or empty($originalheight)) {
         return false;
     }
 
-    $original = imagecreatefromstring(file_get_contents($filepath));
-
     if (function_exists('imagepng')) {
         $imagefnc = 'imagepng';
         $filters = PNG_NO_FILTER;
@@ -306,41 +350,128 @@ function generate_image_thumbnail($filepath, $width, $height) {
         return false;
     }
 
-    if (function_exists('imagecreatetruecolor') and $CFG->gdversion >= 2) {
-        $thumbnail = imagecreatetruecolor($width, $height);
-        if ($imagefnc === 'imagepng') {
-            imagealphablending($thumbnail, false);
-            imagefill($thumbnail, 0, 0, imagecolorallocatealpha($thumbnail, 0, 0, 0, 127));
-            imagesavealpha($thumbnail, true);
-        }
+    if (empty($height)) {
+        $ratio = $width / $originalwidth;
+    } else if (empty($width)) {
+        $ratio = $height / $originalheight;
     } else {
-        $thumbnail = imagecreate($width, $height);
+        $ratio = min($width / $originalwidth, $height / $originalheight);
     }
 
-    $ratio = min($width / $originalwidth, $height / $originalheight);
-
     if ($ratio < 1) {
-        $targetwidth = floor($originalwidth * $ratio);
-        $targetheight = floor($originalheight * $ratio);
+        $targetwidth    = floor($originalwidth * $ratio);
+        $targetheight   = floor($originalheight * $ratio);
     } else {
-        // do not enlarge the original file if it is smaller than the requested thumbnail size
-        $targetwidth = $originalwidth;
-        $targetheight = $originalheight;
+        // Do not enlarge the original file if it is smaller than the requested thumbnail size.
+        $targetwidth    = $originalwidth;
+        $targetheight   = $originalheight;
     }
 
-    $dstx = floor(($width - $targetwidth) / 2);
-    $dsty = floor(($height - $targetheight) / 2);
+    $canvaswidth = $targetwidth;
+    $canvasheight = $targetheight;
+    $dstx = 0;
+    $dsty = 0;
 
-    imagecopybicubic($thumbnail, $original, $dstx, $dsty, 0, 0, $targetwidth, $targetheight, $originalwidth, $originalheight);
+    if ($forcecanvas) {
+        $canvaswidth = $width;
+        $canvasheight = $height;
+        $dstx = floor(($width - $targetwidth) / 2);
+        $dsty = floor(($height - $targetheight) / 2);
+    }
 
+    if (function_exists('imagecreatetruecolor')) {
+        $newimage = imagecreatetruecolor($canvaswidth, $canvasheight);
+        if ($imagefnc === 'imagepng') {
+            imagealphablending($newimage, false);
+            imagefill($newimage, 0, 0, imagecolorallocatealpha($newimage, 0, 0, 0, 127));
+            imagesavealpha($newimage, true);
+        }
+    } else {
+        $newimage = imagecreate($canvaswidth, $canvasheight);
+    }
+
+    imagecopybicubic($newimage, $original, $dstx, $dsty, 0, 0, $targetwidth, $targetheight, $originalwidth, $originalheight);
+
+    if ($imagefnc === 'imagejpeg') {
+        // Function imagejpeg() accepts less arguments than imagepng() but we need to make $imagefnc accept the same
+        // number of arguments, otherwise PHP8 throws an error.
+        $imagefnc = function($image, $filename, $quality, $unused) {
+            return imagejpeg($image, $filename, $quality);
+        };
+    }
+
+    // Capture the image as a string object, rather than straight to file.
     ob_start();
-    if (!$imagefnc($thumbnail, null, $quality, $filters)) {
+    if (!$imagefnc($newimage, null, $quality, $filters)) {
         ob_end_clean();
         return false;
     }
     $data = ob_get_clean();
     imagedestroy($original);
-    imagedestroy($thumbnail);
+    imagedestroy($newimage);
 
     return $data;
 }
+
+/**
+ * Generates a thumbnail for the given image
+ *
+ * If the GD library has at least version 2 and PNG support is available, the returned data
+ * is the content of a transparent PNG file containing the thumbnail. Otherwise, the function
+ * returns contents of a JPEG file with black background containing the thumbnail.
+ *
+ * @param string $filepath the full path to the original image file
+ * @param int $width the width of the requested thumbnail
+ * @param int $height the height of the requested thumbnail
+ * @return string|bool false if a problem occurs, the thumbnail image data otherwise
+ */
+function generate_image_thumbnail($filepath, $width, $height) {
+    return resize_image($filepath, $width, $height, true);
+}
+
+/**
+ * Generates a thumbnail for the given image string.
+ *
+ * If the GD library has at least version 2 and PNG support is available, the returned data
+ * is the content of a transparent PNG file containing the thumbnail. Otherwise, the function
+ * returns contents of a JPEG file with black background containing the thumbnail.
+ *
+ * @param   string $filedata The image content as a string
+ * @param   int $width the width of the requested thumbnail
+ * @param   int $height the height of the requested thumbnail
+ * @return  string|bool false if a problem occurs, the thumbnail image data otherwise
+ */
+function generate_image_thumbnail_from_string($filedata, $width, $height) {
+    if (empty($filedata) or empty($width) or empty($height)) {
+        return false;
+    }
+
+    // Fetch the image information for this image.
+    $imageinfo = @getimagesizefromstring($filedata);
+    if (empty($imageinfo)) {
+        return false;
+    }
+
+    // Create a new image from the file.
+    $original = @imagecreatefromstring($filedata);
+
+    // Generate the thumbnail.
+    return generate_image_thumbnail_from_image($original, $imageinfo, $width, $height);
+}
+
+/**
+ * Generates a thumbnail for the given image string.
+ *
+ * If the GD library has at least version 2 and PNG support is available, the returned data
+ * is the content of a transparent PNG file containing the thumbnail. Otherwise, the function
+ * returns contents of a JPEG file with black background containing the thumbnail.
+ *
+ * @param   resource $original The image to work on.
+ * @param   array $imageinfo Contains [0] => originalwidth, [1] => originalheight.
+ * @param   int $width The width of the requested thumbnail.
+ * @param   int $height The height of the requested thumbnail.
+ * @return  string|bool False if a problem occurs, the thumbnail image data otherwise.
+ */
+function generate_image_thumbnail_from_image($original, $imageinfo, $width, $height) {
+    return resize_image_from_image($original, $imageinfo, $width, $height, true);
+}
