@@ -270,18 +270,17 @@ class helper {
 
         // Get providers preferences.
         foreach ($providers as $provider) {
-            foreach (array('loggedin', 'loggedoff') as $state) {
-                $linepref = get_user_preferences('message_provider_' . $provider->component . '_' . $provider->name
-                    . '_' . $state, '', $userid);
-                if ($linepref == '') {
-                    continue;
-                }
-                $lineprefarray = explode(',', $linepref);
-                $preferences->{$provider->component.'_'.$provider->name.'_'.$state} = array();
-                foreach ($lineprefarray as $pref) {
-                    $preferences->{$provider->component.'_'.$provider->name.'_'.$state}[$pref] = 1;
-                }
+            $linepref = get_user_preferences('message_provider_' . $provider->component . '_' . $provider->name
+                . '_enabled', '', $userid);
+            if ($linepref == '') {
+                continue;
             }
+            $lineprefarray = explode(',', $linepref);
+            $preferences->{$provider->component.'_'.$provider->name.'_enabled'} = [];
+            foreach ($lineprefarray as $pref) {
+                $preferences->{$provider->component.'_'.$provider->name.'_enabled'}[$pref] = 1;
+            }
+
         }
 
         return $preferences;
@@ -309,14 +308,16 @@ class helper {
      *
      * @param object $user User object.
      * @param bool $iscontact
+     * @param bool $displaytextlabel Instructs whether to display a text label.
      * @return array
      */
-    public static function togglecontact_link_params($user, $iscontact = false) {
+    public static function togglecontact_link_params($user, $iscontact = false, bool $displaytextlabel = true) {
         global $USER;
         $params = array(
             'data-currentuserid' => $USER->id,
             'data-userid' => $user->id,
             'data-is-contact' => $iscontact,
+            'data-display-text-label' => $displaytextlabel,
             'id' => 'toggle-contact-button',
             'role' => 'button',
             'class' => 'ajax-contact-button',
@@ -551,10 +552,7 @@ class helper {
         global $USER, $CFG, $PAGE;
 
         // Early bail out conditions.
-        if (empty($CFG->messaging) || !isloggedin() || isguestuser() || user_not_fully_set_up($USER) ||
-            get_user_preferences('auth_forcepasswordchange') ||
-            (!$USER->policyagreed && !is_siteadmin() &&
-                ($manager = new \core_privacy\local\sitepolicy\manager()) && $manager->is_defined())) {
+        if (empty($CFG->messaging) || !isloggedin() || isguestuser() || \core_user::awaiting_action()) {
             return '';
         }
 
@@ -657,21 +655,29 @@ class helper {
      * If disabled, visibility requires that the user be sharing a course with the searching user, and have a visible profile there.
      * The current user is always returned.
      *
+     * You can use the $userfields parameter to reduce the amount of a user record that is required by the method.
+     * The minimum user fields are:
+     *  * id
+     *  * deleted
+     *  * all potential fullname fields
+     *
      * @param \stdClass $user
+     * @param array $userfields An array of userfields to be returned, the values must be a
+     *                          subset of user_get_default_fields (optional)
      * @return array the array of userdetails, if visible, or an empty array otherwise.
      */
-    public static function search_get_user_details(\stdClass $user) : array {
+    public static function search_get_user_details(\stdClass $user, array $userfields = []) : array {
         global $CFG, $USER;
         require_once($CFG->dirroot . '/user/lib.php');
 
         if ($CFG->messagingallusers || $user->id == $USER->id) {
-            return \user_get_user_details_courses($user) ?? []; // This checks visibility of site and course profiles.
+            return \user_get_user_details_courses($user, $userfields) ?? []; // This checks visibility of site and course profiles.
         } else {
             // Messaging specific: user must share a course with the searching user AND have a visible profile there.
             $sharedcourses = enrol_get_shared_courses($USER, $user);
             foreach ($sharedcourses as $course) {
                 if (user_can_view_profile($user, $course)) {
-                    $userdetails = user_get_user_details($user, $course);
+                    $userdetails = user_get_user_details($user, $course, $userfields);
                     if (!is_null($userdetails)) {
                         return $userdetails;
                     }
