@@ -12,6 +12,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
+/* eslint camelcase: off */
 
 /**
  * JavaScript library for the quiz module.
@@ -25,13 +26,19 @@
 M.mod_quiz = M.mod_quiz || {};
 
 M.mod_quiz.init_attempt_form = function(Y) {
-    M.core_question_engine.init_form(Y, '#responseform');
+    require(['core_question/question_engine'], function(qEngine) {
+        qEngine.initForm('#responseform');
+    });
     Y.on('submit', M.mod_quiz.timer.stop, '#responseform');
-    M.core_formchangechecker.init({formid: 'responseform'});
+    require(['core_form/changechecker'], function(FormChangeChecker) {
+        FormChangeChecker.watchFormById('responseform');
+    });
 };
 
 M.mod_quiz.init_review_form = function(Y) {
-    M.core_question_engine.init_form(Y, '.questionflagsaveform');
+    require(['core_question/question_engine'], function(qEngine) {
+        qEngine.initForm('.questionflagsaveform');
+    });
     Y.on('submit', function(e) { e.halt(); }, '.questionflagsaveform');
 };
 
@@ -72,6 +79,37 @@ M.mod_quiz.timer = {
         M.mod_quiz.timer.preview = preview;
         M.mod_quiz.timer.update();
         Y.one('#quiz-timer-wrapper').setStyle('display', 'flex');
+        require(['core_form/changechecker'], function(FormChangeChecker) {
+            M.mod_quiz.timer.FormChangeChecker = FormChangeChecker;
+        });
+        Y.one('#toggle-timer').on('click', function() {
+            M.mod_quiz.timer.toggleVisibility();
+        });
+    },
+
+    /**
+     * Hide or show the timer.
+     * @param {boolean} whether we are ultimately displaying the timer and disabling the button
+     */
+    toggleVisibility: function(finalShow = false) {
+        var Y = M.mod_quiz.timer.Y;
+        var timer = Y.one('#quiz-time-left');
+        var button = Y.one('#toggle-timer');
+
+        // When time is running out, we show the timer and disable the button.
+        if (finalShow) {
+            timer.show();
+            button.setContent(M.util.get_string('hide', 'moodle'));
+            button.setAttribute('disabled', true);
+            return;
+        }
+
+        timer.toggleView();
+        if (timer.getAttribute('hidden') === 'hidden') {
+            button.setContent(M.util.get_string('show', 'moodle'));
+        } else {
+            button.setContent(M.util.get_string('hide', 'moodle'));
+        }
     },
 
     /**
@@ -109,7 +147,7 @@ M.mod_quiz.timer = {
             if (form.one('input[name=finishattempt]')) {
                 form.one('input[name=finishattempt]').set('value', 0);
             }
-            M.core_formchangechecker.set_form_submitted();
+            M.mod_quiz.timer.FormChangeChecker.markFormSubmitted(input.getDOMNode());
             form.submit();
             return;
         }
@@ -119,6 +157,7 @@ M.mod_quiz.timer = {
             Y.one('#quiz-timer').removeClass('timeleft' + (secondsleft + 2))
                     .removeClass('timeleft' + (secondsleft + 1))
                     .addClass('timeleft' + secondsleft);
+            M.mod_quiz.timer.toggleVisibility(true);
         }
 
         // Update the time display.
@@ -139,6 +178,13 @@ M.mod_quiz.timer = {
     updateEndTime: function(timeleft) {
         var newtimeleft = new Date().getTime() + timeleft * 1000;
 
+        // Timer might not have been initialized yet. We initialize it with
+        // preview = 0, because it's better to take a preview for a real quiz
+        // than to take a real quiz for a preview.
+        if (M.mod_quiz.timer.Y === null) {
+            M.mod_quiz.timer.init(window.Y, timeleft, 0);
+        }
+
         // Only update if change is greater than the threshold, so the
         // time doesn't bounce around unnecessarily.
         if (Math.abs(newtimeleft - M.mod_quiz.timer.endtime) > M.mod_quiz.timer.threshold) {
@@ -238,12 +284,12 @@ M.mod_quiz.nav.init = function(Y) {
 
     // Navigation buttons should be disabled when the files are uploading.
     require(['core_form/events'], function(formEvent) {
-        document.addEventListener(formEvent.types.uploadStarted, function() {
+        document.addEventListener(formEvent.eventTypes.uploadStarted, function() {
             M.mod_quiz.filesUpload.numberFilesUploading++;
             M.mod_quiz.filesUpload.disableNavPanel();
         });
 
-        document.addEventListener(formEvent.types.uploadCompleted, function() {
+        document.addEventListener(formEvent.eventTypes.uploadCompleted, function() {
             M.mod_quiz.filesUpload.numberFilesUploading--;
             M.mod_quiz.filesUpload.disableNavPanel();
         });
